function M=bmmsample(Features,nClusters,varargin)
%bmmsample   Sample Bayesian mixture model (multinomial likelihood/Dirichlet prior). 
%
%   mode=bmmsample(Features,nClusters);
%   mode=bmmsample(Features,nClusters,'Param1',Value1,'Param2',Value2,...)
%
%   Input: Required parameters.
%     - Features: Struct as returned by fext().
%     - nClusters: Number of clusters/segments.
%  
%   Input: Additional parameters:  
%     - beta: Dirichlet prior scatter parameter.
%             Settings: Positive scalar.
%             Default: beta = 2*(histogram mass)
%     - nIterations: Number of iterations to execute.
%                    Settings: Positive integer.
%                    Default: 400.
%     - show: Visualize current cluster assignment as image after
%             each iteration.
%             Settings: 0 (don't show), 1 (show).
%             Default: 1.
%             (Uses the imshow() command, set show=0 for use without
%             image processing toolbox.)
%
%   Output:
%     - mode: Mode of assignments computed from the last nSamples=50
%             iterations of the sampler. (nSamples can be adjusted in
%             additional parameters section of the code.)
%             Use imshow(mode/max(max(mode))); to visualize results.
%
% Reference:
% C.P.Roberts: "Mixtures of distributions: Inference and
% estimation", in W.R.Gilks, S.Richardson and D.J.Spiegelhalter (eds.),
% "Markov Chain Monte Carlo in practice", Chapman & Hall, 1996.
%
% Peter Orbanz, ETH Zurich, 2006  
  
  %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
  %
  %  Parameter settings.
  %
  %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
  
  %
  % Parameters computed from input data:
  %
  histograms=Features.histograms;
  nSites=size(histograms,1);
  nBins=size(histograms,2);
  nCounts=sum(histograms(1,:));  
  
  %
  %  Input parameter defaults:
  %
  beta=2*nCounts;
  nIterations=400;
  lambda=1;
  edges=0;
  show=1;
  for i=1:2:(nargin-2),
    switch varargin{i}
     case 'beta'
      beta=varargin{i+1};
     case 'nIterations'
      nIterations=varargin{i+1};
     case 'show'
      show=varargin{i+1};
    end
  end  

  %
  %  Additional settings:
  %
  
  % The Number of samples (at the end of the chain) used to compute
  % the mode estimate.
  nSamples=50;
  if (nSamples > nIterations),
    error('nIterations must exceed nSamples. Please increase nIterations or adjust nSamples in the code.');
  end
  
  % Prior expectation parameter (same as base measure expectation
  % in multinomial/Dirichlet MDP model).
  % Default is the uniform distribution vector. 
  DP_pi = ones(1,nBins)/nBins;
    
  % Random generator initialization:
  %
  % Uncomment the following lines to reset the generator at the
  % start of the program, if you wish to guarantee reproducable results.
  %rand('state',0);
  %randn('state',0);

  
  %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
  %
  %  Sampling algorithm.
  %
  %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
    
  theta_ast=ones(nClusters,nBins)/nBins;
  S_abs=zeros(nSites,nIterations); 
  n_vec=zeros(1,nClusters);     % no entry for component 0
  S=zeros(nSites,1);
  iteration=0;
  expQ=(exp(log(theta_ast)*histograms')');
  
  %Initial assignment step:
  permutation=randperm(nSites);
  for fixedSite=1:nSites,
    iSite=permutation(fixedSite);
    q=expQ(iSite,:);
    S(iSite)=sample(transpose(q/sum(q)));
    n_vec(S(iSite))=n_vec(S(iSite))+1;
  end
  
  
  while (iteration<nIterations),
    iteration=iteration+1;
    disp(['Iteration = ',num2str(iteration)]);
    permutation=randperm(nSites);
    
    % Assignment step:
    
    for fixedSite=1:nSites,
      iSite=permutation(fixedSite);
      current=S(iSite);
      n_vec(current)=n_vec(current)-1;
      q=n_vec.*expQ(iSite,:);
      S(iSite)=sample(transpose(q/sum(q)));
      n_vec(S(iSite))=n_vec(S(iSite))+1;
    end

    % Parameter update step:
    theta_ast=estimate_pars(beta,DP_pi,histograms,S,size(theta_ast,1));
    % By substituting ESTIMATE_PARS_MAP for ESTIMATE_PARS (the
    % arguments of the function call remain unchanged), you
    % can use a MAP estimate instead of a sampling estimate.
    % This will somewhat stabilize the trajectory of the sampler,
    % but also make it harder to get out of local minima. (With a
    % MAP step, the only difference between the sampler and a MAP-EM
    % algorithm is that class assignments are sampled.)
    % Both estimation functions are defined below.
    
    % Recompute cross-entropies for new cluster parameters:
    expQ=(exp(log(theta_ast)*histograms')');

    assignments(:,iteration)=S;    
    if (show),
      imshow(reshape(S,Features.gridRows,Features.gridCols)/max(S));
      colormap(jet)
      drawnow;
    end
  end
  
  M=getmode(assignments(:,end-nSamples+1:end),size(histograms,1),1);

  
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

function qTilde_0=initialize_sampler(histograms,beta,DP_pi)
% Compute the integral \int F()G()d\theta for each histogram.
% Output is a column vector of size nSites x 1.
  disp('Initialize');
  histmass=sum(histograms(1,:));
  nSites=size(histograms,1);
  BP=beta*repmat(DP_pi,nSites,1);
  A=histograms+BP;
  temp_mat=gammaln(A);
  denom_Z_D=gammaln(histmass+beta);
  log_Z_D=sum(temp_mat,2)-denom_Z_D;
  log_Z_D_base=sum(gammaln(beta.*DP_pi))-gammaln(beta);
  qTilde_0 = exp(log_Z_D - log_Z_D_base);

function theta=estimate_pars(beta,DP_pi,histograms,S,nClusters)
% Output theta is nClusters x nBins (no nFrames-dimension!)
% If number of clusters has changed, split()/merge() should
% have taken care of adjusting size of theta_ast.
  nBins=size(histograms,2);
  theta=zeros(nClusters,nBins);
  for k=1:nClusters,
    indicator=(S==i);
    a=sum(repmat(indicator,1,nBins).*histograms,1);
    assigned_indices=find(S==k);
    if (size(assigned_indices)>0),
      theta(k,:)=dirichlet_sample(beta*DP_pi+sum(histograms(assigned_indices,:)));
    end
  end

function theta=estimate_pars_MAP(beta,DP_pi,histograms,S,nClusters)
% Substitute for ESTIMATE_PARS, which replcaces the
% sampling step by a maximum a posteriori point estimate.
  nBins=size(histograms,2);
  theta=zeros(nClusters,nBins);
  for k=1:nClusters,
    assigned_indices=find(S==k);
    if (size(assigned_indices)>0),
      theta(k,:)=beta*DP_pi+sum(histograms(assigned_indices,:));
      theta(k,:)=theta(k,:)/sum(theta(k,:));
    end
  end

% The sampling function for the Dirichlet distribution
% is a modified version of the one provided by Tom Minka's 
% "Fastfit" package, available at 
% http://research.microsoft.com/~minka/software/fastfit/
function r = dirichlet_sample(a,n)
  if nargin < 2
    n = 1;
  end
  row = (size(a,1) == 1);
  a = a(:);
  y = gamrnd(repmat(a, 1, n),1);
  % randgamma is faster
  %y = randgamma(repmat(a, 1, n));
  r = sum(y,1);
  r(find(r == 0)) = 1;
  r = y./repmat(r, size(y,1), 1);
  if row
    r = r';
  end  
  
function M=getmode(S_history,gridRows,gridCols)
% Computes a mode estimate of assignments by
% simple maximization: Each point is assigned
% to the cluster present in the last sample
% which it was most often assigned to.
% Clusters not present in the last sample are
% not taken into account.
  nSites=size(S_history,1);
  last=S_history(:,end);
  % Determine clusters present in last sample:
  MIN=min(last);
  MAX=max(last);
  clusters=zeros(1,0);
  for k=MIN:MAX,
    if (~isempty(find(last==k))),
      clusters=addcolumn(clusters);
      clusters(1,end)=k;
    end
  end
  % Compute assignment probabilities:
  nClusters=size(clusters,2);
  probs=zeros(nSites,nClusters);
  for k=1:size(clusters,2),
    probs(:,k)=sum(S_history==clusters(k),2);
  end
  [dummy,M]=max(probs,[],2);
  M=reshape(M,gridRows,gridCols);
