function features=fext(input_image,nBins,meshsize,windowrad,varargin)
%fext  Histogram feature extraction.
%
%   Features=fext(input_image,nBins,meshsize,windowrad)
%   Features=fext(input_image,nBins,meshsize,windowrad,edgemap)
%
%   Input:
%     - input_image: Grayscale image.
%     - nBins: Number of histogram bins.
%     - meshsize: Distance (in pixels) between nodes of the
%                 neighborhood graph.
%     - windowrad: Radius of the histogram window 
%                  (window size = (2*WINDOWRAD+1)^2 pixels)
%     - edgemap: A 0-1 edgemap, same size as INPUT_IMAGE. 1-entries
%                are interpreted as edge locations. If call passes an 
%                edgemap, a second neighborhood graph is computed with 
%                connections crossing image edges removed.
%
%   Article defaults: nBins=8, meshsize=2, windowrad=2.
%
%   Output: Struct Features, fields:
%     - histograms: Matrix (nSites x nBins) of local histograms.
%     - nHoodList_noedges: MRF neighborhood list (no edge
%                          information).
%     - nHoodList_edges: MRF neighborhood list (using edge
%                        information, returned only if input
%                        contains edgemap)
%     - gridRows: Number of vertical site positions.
%     - gridCols: Number of horizontal site positions.
%
% Peter Orbanz, ETH Zurich, 2006
  
  edges=0;
  threshold=1;
  if (nargin>4),
    edgemap=varargin{1};
    edges=1;
    if (nargin>5),
      threshold=varargin{2};
    end
  end
  
  padded_image=ffext_pad_image(input_image,windowrad);
    
  [histograms,nRows,nCols]=ffext_histograms(padded_image,nBins,meshsize,windowrad);
  nhoodlist_noedges=ffext_nhoodlist(padded_image,meshsize,windowrad);
  if (edges),
    padded_edgemap=ffext_pad_image(edgemap,windowrad);
    indices=ffext_edgeindices(padded_edgemap,meshsize,windowrad, ...
			      threshold);
    nhoodlist_edges=nhoodlist_noedges;
    nhoodlist_edges(indices,:)=0;
  end
  
  gridRows=nRows/meshsize;
  gridCols=nCols/meshsize;
  
  if (~edges),
    features=struct('histograms',histograms,'nHoodList_noedges',nhoodlist_noedges,'gridRows',gridRows,'gridCols',gridCols);
  else
    features=struct('histograms',histograms,'nHoodList_noedges',nhoodlist_noedges,'nHoodList_edges',nhoodlist_edges,'gridRows',gridRows,'gridCols',gridCols);
  end

  
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%
%  Auxiliary functions.
%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

function [histograms,nWindowsVertical,nWindowsHorizontal]=ffext_histograms(image,nBins,meshsize,windowrad)
  
  nWindowsVertical=size(image,1)-2*windowrad;
  nWindowsHorizontal=size(image,2)-2*windowrad;
  windowsize=2*windowrad+1;
    
  A=im2col(image,[windowsize windowsize],'sliding')';
  B=reshape(A,nWindowsVertical,nWindowsHorizontal,windowsize^2);
  B=B(windowrad:meshsize:end,windowrad:meshsize:end,:);
  B=reshape(B,size(B,1)*size(B,2),windowsize^2);
  histograms=hist(double(B)',nBins)';


function padded_image=ffext_pad_image(input_image,windowrad)
  padded_image=zeros(size(input_image)+[2*windowrad 2*windowrad]);
  %Upper:
  padded_image(1:windowrad,1+windowrad:end-windowrad)=input_image(1:windowrad,:);
  %Lower:
  padded_image(end-windowrad+1:end,1+windowrad:end-windowrad)= ...
      input_image(end-windowrad+1:end,:);
  %Left:
  padded_image(1+windowrad:end-windowrad,1:windowrad)=input_image(:,1:windowrad);
  %Right:
  padded_image(1+windowrad:end-windowrad,end-windowrad+1:end)=input_image(:,end-windowrad+1:end);
  %Corner blocks:
  %Upper left:
  padded_image(1:1+windowrad,1:1+windowrad)=input_image(1:1+windowrad,1:1+windowrad);
  %Lower left:
  padded_image(end-windowrad+1:end,1:1+windowrad)=input_image(end-windowrad+1:end,1:1+windowrad);
  %Upper right:
  padded_image(1:1+windowrad,end-windowrad+1:end)=input_image(1:1+windowrad,end-windowrad+1:end);
  %Lower right:
  padded_image(end-windowrad+1:end,end-windowrad+1:end)=input_image(end-windowrad+1:end,end-windowrad+1:end);
  %Main image:
  padded_image(1+windowrad:end-windowrad,1+windowrad:end-windowrad)=input_image;


function indices=ffext_edgeindices(edgemap,meshsize,windowrad,threshold)
% threshold: default should be 1
% site is edgesite iff number of edgepixels >= threshold
  nWindowsVertical=size(edgemap,1)-2*windowrad;
  nWindowsHorizontal=size(edgemap,2)-2*windowrad;
  windowsize=2*windowrad+1;
  A=im2col(edgemap,[windowsize windowsize],'sliding')';
  B=reshape(A,nWindowsVertical,nWindowsHorizontal,windowsize^2);
  B=B(windowrad:meshsize:end,windowrad:meshsize:end,:);
  B=reshape(B,size(B,1)*size(B,2),windowsize^2);
  B=sum(B,2);
  indices=find(B>=1);
  
function list=ffext_nhoodlist(image,meshsize,windowrad)
  nWindowsVertical=size(image,1)-2*windowrad;
  nWindowsHorizontal=size(image,2)-2*windowrad;
  windowsize=2*windowrad+1;
  nSites=nWindowsVertical*nWindowsHorizontal/meshsize^2;
  indexVec=1:nSites;
  indexMat=reshape(indexVec,nWindowsVertical/meshsize,nWindowsHorizontal/meshsize);
  paddedMat=zeros(size(indexMat)+[2 2]);
  paddedMat(2:end-1,2:end-1)=indexMat;
  list=im2col(paddedMat,[3 3],'sliding')';
  %5th column (middle column) always references site itself, so
  %remove it:
  list=[list(:,1:4) list(:,6:9)];
