function M=mdpmrfsample(Features,alpha,varargin)
%mdpmrfsample   Sample MDP/MRF model.
%
%   mode=mdpmrfsample(Features,alpha);
%   mode=mdpmrfsample(Features,alpha,'Param1',Value1,'Param2',Value2,...)
%
%
%   Input: Required parameters.
%     - Features: Struct as returned by fext().
%     - alpha: DP hyperparameter alpha (positive scalar).
%  
%   Input: Additional parameters:  
%     - beta: Dirichlet base measure scatter parameter.
%             Settings: Positive scalar.
%             Default: beta = 2*(histogram mass)
%     - lambda: MRF smoothing parameter. 
%               Settings:  0 = no smoothing, 1 = standard smoothing
%               Default: lambda = 1.
%               Increase lambda to increase influence of MRF smoothing.  
%     - edges: Toggle use of edge information (no effect for lambda=0).
%              Settings: 0 (no edge information), 1 (use edge information).
%              Default: 0.
%     - nIterations: Number of iterations to execute.
%                    Settings: Positive integer.
%                    Default: 400.
%     - show: Visualize current cluster assignment as image after
%             each iteration.
%             Settings: 0 (don't show), 1 (show).
%             Default: 1.
%             (Uses the imshow() command, set show=0 for use without
%             image processing toolbox.)
%
%   Output:
%     - mode: Mode of assignments computed from the last nSamples=50
%             iterations of the sampler. (nSamples can be adjusted in
%             additional parameters section of the code.)
%             Use imshow(mode/max(max(mode))); to visualize results.
%
% Peter Orbanz, ETH Zurich, 2006 

  
  %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
  %
  %  Parameter settings.
  %
  %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
  
  %
  % Parameters computed from input data:
  %
  histograms=Features.histograms;
  nSites=size(histograms,1);
  nBins=size(histograms,2);
  nCounts=sum(histograms(1,:));  
  
  %
  %  Input parameter defaults:
  %
  beta=2*nCounts;
  nIterations=400;
  lambda=1;
  edges=0;
  show=1;
  for i=1:2:(nargin-2),
    switch varargin{i}
     case 'beta'
      beta=varargin{i+1};
     case 'lambda'
      lambda=varargin{i+1};
     case 'nIterations'
      nIterations=varargin{i+1};
     case 'edges'
      edges=varargin{i+1};
     case 'show'
      show=varargin{i+1};
    end
  end  
  if (edges),
    neighborList=Features.nHoodList_edges;
  else
    neighborList=Features.nHoodList_noedges;
  end
  
  %
  %  Additional settings:
  %
  
  % The Number of samples (at the end of the chain) used to compute
  % the mode estimate.
  nSamples=50;
  if (nSamples > nIterations),
    error('nIterations must exceed nSamples. Please increase nIterations or adjust nSamples in the code.');
  end
  
  % Base meausure expectation parameter pi: Default is the uniform
  % distribution vector. If adapting to the data is desirable, consider
  % setting pi to the average normalized input histogram by
  % exchanging the lines below.
  % 
  % Uniform:
  DP_pi = ones(1,nBins)/nBins;
  % Average normalized input histogram:
  %DP_pi = sum(histograms,1)./nCounts;
  
  % Initialization of the clusters:
  %
  % Start with a single cluster containing all data (default, value 1)
  % or with zero clusters (value 0)?
  % Zero cluster means the first cluster is created when the first
  % data point is examined during the initial iteration. This will
  % result in higher variability of the results.
  singleClusterInit=1;
  
  % Number of iterations before MRF is activated: 
  %
  % If this is larger
  % than zero, the algorithm will, strictly speaking, sample two different
  % processes (first a standard DP, then a smoothed DP when MRF ist
  % activated).
  %
  nInitialIterations=0;
  %
  if (singleClusterInit == 0 & nInitialInterations == 0 ),
    disp('Error: When initializing with zero clusters, nInitialIterations must be at least 1.');
  end
  
  % Random generator initialization:
  %
  % Uncomment the following lines to reset the generator at the
  % start of the program, if you wish to guarantee reproducable results.
  %rand('state',0);
  %randn('state',0);

  
  
  
  %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
  %
  %  Sampling algorithm.
  %
  %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

  nTotalClusters=0;
  nClusters=0;
  theta_ast=zeros(nClusters,nBins);
  S_abs=zeros(nSites,nIterations); 
  n_vec=zeros(nTotalClusters,1);     % no entry for component 0
  S=zeros(nSites,1);
  if (singleClusterInit),
    n_vec=[0 nSites];
    S=ones(nSites,1);
    nTotalClusters=1;
    nClusters=1;
    theta_ast=dirichlet_sample(beta*DP_pi+sum(histograms,1));
    log_theta=log(theta_ast);
    expQ=(exp(log_theta*histograms')');
    abs_index=[1];
  else
    nInitialIterations=1;
    n_vec=nSites;
    S=zeros(nSites,1);
    nTotalClusters=0;
    nClusters=0;
    expQ=zeros(nSites,0);
    abs_index=[];
  end
  % Compute the normalization constants
  qTilde_0=alpha*initialize_sampler(histograms,beta,DP_pi);
  iteration=0;

  if (show),
    figure
  end
  
  while (iteration<nIterations),
    iteration=iteration+1;
    disp(['Iteration = ',num2str(iteration),' nClusters = ' ...
	  ' ',num2str(nClusters),' nTotalClusters' ...
	  ' = ',num2str(nTotalClusters)]);
    permutation=randperm(nSites);
    if (lambda > 0 & iteration == nInitialIterations + 1),
      disp('Smoothing activated.');
    end
    % Using an all-zeros neighborList effectively switches 
    % off MRF term during initial iterations.
    if (lambda > 0 & iteration > nInitialIterations),
      local_neighborList=neighborList;
    else
      local_neighborList=zeros(size(neighborList));
    end
    
    % Assignment step:
    
    for fixedSite=1:nSites,
      iSite=permutation(fixedSite);
      current=S(iSite);
      n_vec(current+1)=n_vec(current+1)-1;
      nlist_local=local_neighborList(iSite,:);
      neighbors=nlist_local(find(nlist_local));
      log_MRFC=zeros(1,nClusters);
      for iNeighbor=1:length(neighbors),
	log_MRFC(S(neighbors(iNeighbor)))=log_MRFC(S(neighbors(iNeighbor)))+lambda;
      end
      q=[qTilde_0(iSite) n_vec(2:end).*expQ(iSite,:).*exp(log_MRFC)];
      S(iSite)=sample(transpose(q/sum(q)))-1;
      if (S(iSite)==0),
	n_vec=[n_vec 0];
	theta_ast=[theta_ast; dirichlet_sample(histograms(iSite,:)+beta*DP_pi)];
	nClusters=nClusters+1;
	S(iSite)=nClusters;
	log_theta=log(theta_ast(end,:));
	expQ=[expQ (exp(log_theta*histograms')')];
	nTotalClusters=nTotalClusters+1;
	abs_index=[abs_index nTotalClusters];
      end
      n_vec(S(iSite)+1)=n_vec(S(iSite)+1)+1;
    end

    % Clean up: Remove clusters run empty.
    remove_indices=find(n_vec(2:end)==0);
    if (length(remove_indices)>1),
      disp(['Merge: Remove ',num2str(length(remove_indices)),' clusters']);
    end
    for k=length(remove_indices):-1:1,
      n_vec(remove_indices(k)+1)=n_vec(end);
      theta_ast(remove_indices(k),:)=theta_ast(end,:);
      expQ(:,remove_indices(k))=expQ(:,end);
      abs_index(remove_indices(k))=abs_index(end);
      S(find(S==nClusters))=remove_indices(k);
      n_vec=n_vec(1:end-1);
      theta_ast=theta_ast(1:end-1,:);
      expQ=expQ(:,1:end-1);
      abs_index=abs_index(1:end-1);
      nClusters=size(theta_ast,1);
    end
    
    % Parameter update step:
    theta_ast=estimate_pars(beta,DP_pi,histograms,S,size(theta_ast,1));
    % By substituting ESTIMATE_PARS_MAP for ESTIMATE_PARS (the
    % arguments of the function call remain unchanged), you
    % can use a MAP estimate instead of a sampling estimate.
    % This will somewhat stabilize the trajectory of the sampler,
    % but also make it harder to get out of local minima. (With a
    % MAP step, the only difference between the sampler and a MAP-EM
    % algorithm is that class assignments are sampled.)
    % Both estimation functions are defined below.
    
    % Recompute cross-entropies for new cluster parameters:
    expQ=(exp(log(theta_ast)*histograms')');

    assignments(:,iteration)=S;    
    if (show),
      imshow(reshape(S,Features.gridRows,Features.gridCols)/max(S));
      colormap(jet)
      drawnow;
    end
  end
  
  M=getmode(assignments(:,end-nSamples+1:end),Features.gridRows,Features.gridCols);
  
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%
%  Auxiliary functions
%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

function qTilde_0=initialize_sampler(histograms,beta,DP_pi)
% Compute the integral \int F()G()d\theta for each histogram.
% Output is a column vector of size nSites x 1.
  disp('Initialize');
  histmass=sum(histograms(1,:));
  nSites=size(histograms,1);
  BP=beta*repmat(DP_pi,nSites,1);
  A=histograms+BP;
  temp_mat=gammaln(A);
  denom_Z_D=gammaln(histmass+beta);
  log_Z_D=sum(temp_mat,2)-denom_Z_D;
  log_Z_D_base=sum(gammaln(beta.*DP_pi))-gammaln(beta);
  qTilde_0 = exp(log_Z_D - log_Z_D_base);

function theta=estimate_pars(beta,DP_pi,histograms,S,nClusters)
% Output theta is nClusters x nBins (no nFrames-dimension!)
% If number of clusters has changed, split()/merge() should
% have taken care of adjusting size of theta_ast.
  nBins=size(histograms,2);
  theta=zeros(nClusters,nBins);
  for k=1:nClusters,
    indicator=(S==i);
    a=sum(repmat(indicator,1,nBins).*histograms,1);
    assigned_indices=find(S==k);
    if (size(assigned_indices)>0),
      theta(k,:)=dirichlet_sample(beta*DP_pi+sum(histograms(assigned_indices,:)));
    end
  end
  
function theta=estimate_pars_MAP(beta,DP_pi,histograms,S,nClusters)
% Substitute for ESTIMATE_PARS, which replcaces the
% sampling step by a maximum a posteriori point estimate.
  nBins=size(histograms,2);
  theta=zeros(nClusters,nBins);
  for k=1:nClusters,
    assigned_indices=find(S==k);
    if (size(assigned_indices)>0),
      theta(k,:)=beta*DP_pi+sum(histograms(assigned_indices,:));
      theta(k,:)=theta(k,:)/sum(theta(k,:));
    end
  end

% The sampling function for the Dirichlet distribution
% is a modified version of the one provided by Tom Minka's 
% "Fastfit" package, available at 
% http://research.microsoft.com/~minka/software/fastfit/
% The multinomial sampling function sample() is taken
% from the same package.
function r = dirichlet_sample(a,n)
  if nargin < 2
    n = 1;
  end
  row = (size(a,1) == 1);
  a = a(:);
  y = gamrnd(repmat(a, 1, n),1);
  % randgamma is faster
  %y = randgamma(repmat(a, 1, n));
  r = sum(y,1);
  r(find(r == 0)) = 1;
  r = y./repmat(r, size(y,1), 1);
  if row
    r = r';
  end  

function x = sample(p,n)
  if nargin < 2
    n = 1;
  end
  cdf = cumsum(p(:));
  if cdf(end) <= 0
    error('distribution is all zeros');
  end
  for i = 1:n
    x(i) = sum(cdf < rand*cdf(end)) + 1;
  end

function M=getmode(S_history,gridRows,gridCols)
% Computes a mode estimate of assignments by
% simple maximization: Each point is assigned
% to the cluster present in the last sample
% which it was most frequently assigned to.
% Clusters not present in the last sample are
% not taken into account.
  nSites=size(S_history,1);
  last=S_history(:,end);
  % Determine clusters present in last sample:
  MIN=min(last);
  MAX=max(last);
  clusters=zeros(1,0);
  for k=MIN:MAX,
    if (~isempty(find(last==k))),
      clusters=[clusters zeros(size(clusters,1),1)];
      clusters(1,end)=k;
    end
  end
  % Compute assignment probabilities:
  nClusters=size(clusters,2);
  probs=zeros(nSites,nClusters);
  for k=1:size(clusters,2),
    probs(:,k)=sum(S_history==clusters(k),2);
  end
  [dummy,M]=max(probs,[],2);
  M=reshape(M,gridRows,gridCols);
