function [c, a] = calc_dissimilarity(Y1, Y2, k)
% CALC_DISSIMILARITY  Calculate dissimilarity of two labellings
%
%   [c, a] = calc_dissimilarity(Y1, Y2, k) calculates the dissimilarity of
%   two solutions of a k-clustering problem where Y1 and Y2 are the
%   label-vectors of these two solutions.
%   
%   To calculate the dissimilarity a maximum bipartite matching is computed
%   between Y1 and Y2. Its normalized cost (the dissimilarity) is returned
%   in c and the assignement itself is returned in a.
%
%   To get the permuted label vector Y2s from the assignement a use the
%   following code:
%
%       n = length(Y2);
%       [c, a] = calc_dissimilarity(Y1, Y2, k);
%       A = repmat(a, n, 1);
%       Y2s = A(sub2ind(size(A), (1:n)', Y2));
%
%   See also STABILITY_INDEX
%
%   NOTE: This code uses an implementation of the hungarian
%   method written by Niclas Borlin, niclas@cs.umu.se.
%   Department of Computing Science, Ume University, Sweden. 

%   Reference: Lange, Roth, Braun, and Buhmann, 
%   "Stability-Based Validation of Clustering Solutions",
%	Neural Computation, 16: 1299-1323, June, 2004, MIT Press
%   
%   Original coding by Simon Meier
%   Copyright 2005 Swiss Federal Institute of Technology (ETH) Zurich
%   Institute for Computational Science
%   $Revision: 1.2 $ $Date: 2005/06/28 23:31:12 $



% init
n = size(Y1,1);

% compute the weight matrix W
W = zeros(k,k);
for J = 1:n
    W(Y1(J), Y2(J)) = W(Y1(J), Y2(J)) + 1;
end;

% and using the hungarian method to compute a maximum weight matching
% in a bipartite graph (where c is the matching and d its cost)
[a, c] = hungarian(-W);
c = 1 + c/n;


