function [S, Svalues] = stability_index(Features,nClusters,beta,nIterations, cluster_alg, classifier, r, subsampler, s)
% STABILITY_INDEX   Calculates the stability index of a clustering solution
%   
%   [S, Svalues] = stability_index(X, k, cluster_alg, classifier, r,
%                                  subsampler, s)
%
%   Calculates the stability index S of the given clustering problem.  That
%   means, to partition the data X into k disjoint classes or clusters.
% 
%   The data X is given as a matrix where each row represents one datum.
% 
%   The clustering algorithm is given by the function handle cluster_alg.
%   This function must have the following format:
% 
%       Y = cluster_alg(X, k)
% 
%   Where Y is a vector of labels out of {1..k} corresponding to the given
%   data X.
% 
%   The classifier that is used to make the two solutions of the subsamples
%   comparable must have the following format:
% 
%       Y2 = classifier(X1, Y1, X2)
% 
%   where Y2 are the labels of the data X2 if the given data X1 with
%   corresponding labels Y1 is used to train the classifier. To generate
%   such a function directly from given training and classification
%   functions see STAB_CLASSIFIER.
% 
%   Because the the stability index is given by taking the expectation over
%   all possible independent datasets X1 and X2, the stability index is
%   approximated by averaging over the result of r supsamples generated by
%   the subsampler out of X. The subsampler must have the following format:
% 
%       [X1, X2, newState] = subsampler(X, lastState)
% 
%   where X1 and X2 are two subsamples generated from the data X and the
%   state lastState. The state in the first call of subsampler will be []. 
%   For a default subsampler see RND_SPLIT_SUBSAMPLE.
% 
%   To normalize and therefore make the stability indices of the clustering
%   problems with a different number of clusters comparable, the following
%   approach is chosen: s random k-labelings of the data are sampled. The
%   empirical average of the dissimilarities of these k-labelings is used
%   to normalize the stability index.
% 
%   The column vector Svalues contains all r normalized values of stability
%   indices that occured during the computation.
% 
% 
%   Examples
%       see DEMO_STAB_1 and DEMO_STAB_2
%
%   See also MOST_STABLE_CLUSTER_MODEL, STAB_CLASSIFIER

%   Reference: Lange, Roth, Braun, and Buhmann, 
%   "Stability-Based Validation of Clustering Solutions",
%	Neural Computation, 16: 1299-1323, June, 2004, MIT Press
%   
%   Original coding by Simon Meier
%   Copyright 2005 Swiss Federal Institute of Technology (ETH) Zurich
%   Institute for Computational Science
%   $Revision: 1.2 $ $Date: 2005/06/28 23:31:12 $


X=Features.histograms;
% init
n = size(X,1);
k=nClusters;

% perform r splits
subsampler_state = [];
Svalues = [];
for I = 1:r
    % get subsample
    [X1, X2, subsampler_state] = subsampler(X, subsampler_state);
    
    F1=Features;
    F1.histograms=X1;
    F2=Features;
    F2.histograms=X2;
    
    % cluster both
    Y1=cluster_alg(F1,nClusters,beta,nIterations);
    Y2_1=cluster_alg(F1,nClusters,beta,nIterations);

    % classify X2 according to the classifier trained on (X1, Y1)
    Y2_2 = classifier(X1, Y1, X2);
    
    % and calc dissimilarity:
    S = calc_dissimilarity(Y2_1, Y2_2, k);
    
    Svalues = [Svalues; S];
end

% calc normalization
norm_err = 0;
n_half = ceil(n/2);

for I = 1:s
    Y2_1 = ones(n_half, 1);
    Y2_2 = 1+floor(k*rand(n_half,1));
    norm_err = norm_err + calc_dissimilarity(Y2_1, Y2_2, k);
end;
norm_err = norm_err / s;

% normalize and return
Svalues = Svalues / norm_err;
S = mean(Svalues);



